;; Generated package description from gnu-elpa-keyring-update.el
(define-package "gnu-elpa-keyring-update" "2019.2" "Update Emacs's GPG keyring for GNU ELPA" 'nil :url "http://elpa.gnu.org/packages/gnu-elpa-keyring-update.html" :keywords '("maint" "tools"))
